function html = FlouLibHelp(fileStr)
% DSPBHELP DSP Blockset on-line help function.
%   Points Web browser to the HTML help file corresponding to this
%   DSP Blockset block.  The current block is queried for its MaskType.
%
%   Typical usage:
%      set_param(gcb,'MaskHelp','web(FlouLibHelp);');

% Copyright (c) 1995-1999 The MathWorks, Inc. All Rights Reserved.
% $Revision: 1.13 $ $Date: 1999/01/13 13:58:13 $

error(nargchk(0,1,nargin));
d = docroot;

if isempty(d),
   % Help system not present:
   %   html = ['file:///',FlouLibPath,'\help\' matlabroot '/toolbox/dspblks/dspblks/dspbherr.html'];
   print "ici"
   web ['file:///',FlouLibPath,'\help\FlouLibRef.html'];
else
   if nargin < 1
      % Derive help file name from mask type:
      html_file = getblock_help_file(gcb);
   else
      % Derive help file name from fileStr argument:
      html_file = getarg_help_file(fileStr);
   end
   
   % Construct full path to help file.
   % Use 3 forward slashes for portability of HTML file paths:
   html = ['file:///',FlouLibPath,'\help\' html_file];

end
return

% --------------------------------------------------------
function html_file = getarg_help_file(fileStr)

html_file = help_name(fileStr);
return


% --------------------------------------------------------
function help_file = getblock_help_file(blk)

linkStatus = lower(get_param(blk,'LinkStatus'));

switch linkStatus
case 'resolved'
   rblock = get_param(blk,'ReferenceBlock');
   sys = get_param(rblock,'Parent');
case 'none'
   sys = get_param(blk,'Parent');
otherwise
   error('Unknown link status reported.');
end



   % Only masked DSP Blockset blocks call dspbhelp, so if
   % we get here, we know we can get the MaskType string.
   fileStr = get_param(blk,'MaskType');

help_file = help_name(fileStr);

return

% ---------------------------------------------------------
function y = help_name(x)
% Returns proper help-file name
%
% Invoke same naming convention as used with the
% auto-generated help conversions for the blockset
% on-line manuals.
%
% - only allow a-z, 0-9, and underscore
% - truncate to 25 chars max, plus ".html"

if isempty(x), x='default'; end
y = lower(x);
y(find(~isvalidchar(y))) = '';  % Remove invalid characters
if length(y)>25, y=y(1:25); end
y = [y '.html'];

return


% ---------------------------------------------------------
function y = isvalidchar(x)
y = isletter(x) | isdigit(x) | isunder(x);
return


% ---------------------------------------------------------
function y = isdigit(x)
y = (x>='0' & x<='9');
return


% ---------------------------------------------------------
function y = isunder(x)
y = (x=='_');
return


% [EOF] dspbhelp.m
