%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% FlouLib installation script
%
% Copyright: Universit de Savoie, LISTIC/ESIA.
%
% Author: L. Foulloy
%
% Revision: V3_2 - L. Foulloy - 25/10/2004
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Install(Param)

if nargin<1
   Param = '';
end

%User Blocks if needed. The format of a block is Name, Type, FileName, Optional associated files.
% Modify and/or add new lines to add your own blocks.
UserBlock(1) 					= NewBlock('MyFunctionName1', 	'MyFunctionClass',      '',    '');
UserBlock(length(UserBlock)+1)	= NewBlock('MyFunctionName2', 	'MyFunctionClass',      '',    '');

%FlouLib Blocks : DO NOT CHANGE
%Fuzzification
Block(1) 						= NewBlock('SymbFuz',           'Fuzzification',		'SymbFuz.c',            '');
Block(length(Block)+1) 			= NewBlock('SymbFuzSup', 		'Fuzzification',		'SymbFuzSup.c',         '');
Block(length(Block)+1) 			= NewBlock('SymbFuzInf', 		'Fuzzification',		'SymbFuzInf.c',         '');
Block(length(Block)+1) 			= NewBlock('SymbFuz2D', 		'Fuzzification',		'SymbFuz2D.c',          '');
Block(length(Block)+1) 			= NewBlock('NumFuzTriangle', 	'Fuzzification',		'NumFuzTriangle.c',     '');

%Inference
Block(length(Block)+1) 			= NewBlock('SymbInfMam', 		'Inference', 			'SymbInfMam.c',         'UserOperators.c');
Block(length(Block)+1) 			= NewBlock('SymbInfSug', 		'Inference', 			'SymbInfSug.c',         '');
Block(length(Block)+1) 			= NewBlock('SymbInfSugAdap', 	'Inference', 			'SymbInfSugAdap.c',     '');

%Defuzzification
Block(length(Block)+1) 			= NewBlock('SymbDefuz', 		'Defuzzification', 	    'SymbDefuz.c',          'UserDefuzMethods.c');

%Fuzzy Controllers
Block(length(Block)+1) 			= NewBlock('SugenoFLC', 		'Fuzzy Controller', 	'SugenoFLC.c',          '');
Block(length(Block)+1) 			= NewBlock('MamdaniFLC', 		'Fuzzy Controller', 	'MamdaniFLC.c',         'UserOperators.c UserDefuzMethods.c');

%Plots
Block(length(Block)+1) 			= NewBlock('PlotPartition', 	'Display', 				'PlotPartition.c',      '');
Block(length(Block)+1) 			= NewBlock('PlotPartition2D',	'Display', 				'PlotPartition2D.c',    '');
Block(length(Block)+1) 			= NewBlock('Plot',				'Display', 				'PlotResult.m',         '');
Block(length(Block)+1) 			= NewBlock('PlotSurface',		'Display', 				'PlotSurface.m',        '');

%Operators

Block(length(Block)+1) 			= NewBlock('SumInputs', 		'Operators', 			'SumInputs.c',          '');
Block(length(Block)+1) 			= NewBlock('CompInputs', 		'Operators', 			'CompInputs.c',         'UserCompMethods.c');
Block(length(Block)+1) 			= NewBlock('FuzzyOp', 			'Operators', 			'FuzzyOp.c',            'UserOperators.c');
Block(length(Block)+1) 			= NewBlock('FuzzyOpEq', 		'Operators', 			'FuzzyOpEq.c',          '');
	
%Various
Block(length(Block)+1) 			= NewBlock('RuleBaseGenerator', 'Various', 				'RuleSugMEP.c',    		'');
Block(length(Block)+1) 			= NewBlock('FISParser', 		'Various', 				'ParsFis.m', 		   	'');

%Special Blocks for separate compilation
ObjBlock(1) 					= NewBlock('SymbInfMam', 		'Inference', 			'SymbInfMam.c',         '');
ObjBlock(length(ObjBlock)+1)	= NewBlock('SymbDefuz', 		'Defuzzification', 	    'SymbDefuz.c',          '');
ObjBlock(length(ObjBlock)+1)	= NewBlock('CompInputs', 		'Operators', 			'CompInputs.c',         '');
ObjBlock(length(ObjBlock)+1)	= NewBlock('FuzzyOp', 			'Operators', 			'FuzzyOp.c',            '');
ObjBlock(length(ObjBlock)+1)	= NewBlock('MamdaniFLC', 		'Fuzzy Controller', 	'MamdaniFLC.c',         '');

switch Param
  case '-h'
     if exist('Version.txt')
        disp('Options: -Update -Ref, -Path, -Help, -ClearFiles');
        disp('  -Compile   : Compile the files using mex');
        disp('  -Update    : Use this option each time you change one UserXXXXX.c file (Operators, CompMethods, DefuzMethods');
        disp('  -Ref       : Generate FlouLibRef.html file');
        disp('  -Path      : Generate FlouLibPath.m file');
        disp('  -Help      : Generate the help files');
        disp('  -ClearFiles: Clear the .dll and .obj files in the FLouLib directory');
     else
        disp('Options: -Compile, -Update -Ref, -Path, -Help, -ClearFiles -Export');
        disp('  -Compile   : Compile the files using mex');
        disp('  -Update    : Use this option each time you change one UserXXXXX.c file (Operators, CompMethods, DefuzMethods');
        disp('  -Ref       : Generate FlouLibRef.html file');
        disp('  -Path      : Generate FlouLibPath.m file');
        disp('  -Help      : Generate the help files');
        disp('  -ClearFiles: Clear the .dll and .obj files in the FLouLib directory');
        disp('  -Export    : Generate the export version of FlouLib in the Export directory');   
     end   
  case '-Compile'
     % Valid if not in the export version
     if exist('Version.txt')
        disp('Invalid option');
     else
        Compile(UserBlock, '-v'); 
        Compile(ObjBlock, '-v -c'); 
        Compile(Block, '-v');
     end
  case '-Update'
     UpDate(Block,'');
  case '-Ref'
     ModifyFlouLibRef(Block);
  case '-Path'
     ModifyFlouLibPath;
     addpath(pwd);
     ModifyPathDef;
  case '-Help'
     ModifyHelp(Block);
     ModifyFlouLibRef(Block);
  case '-ClearFiles'
     button = questdlg(['You are going to erase all the files *.dll and *.obj from the directory ',pwd,char(10),'Are you sure?'], ...
                   'Continue Operation','Yes','No','No');
     if strcmp(button,'Yes')
        delete '*.obj';
        delete '*.dll';
     else
        return  
     end
  case '-Export'
     % Valid if not in the export version
     if exist('Version.txt')
        disp('Invalid option');
     else
        Compile(ObjBlock, '-c'); 
        Compile(Block, '');
        ExportFlouLib;
     end   
  case ''     
     button = questdlg(['You are going to intall FlouLib. This shell will modify the file ', matlabroot,'\toolbox\local\pathdef.m',char(10), ...
             'A copy of the original file is stored in copy_pathdef.m',char(10),'Do you want to proceed?'], ...
                   'Continue Operation','Yes','No','No');
     if strcmp(button,'No')
        return  
     end
     disp('Starting installation');
     Compile(UserBlock, ''); 
     % Compile the block if not in the export version
     if ~exist('Version.txt')
       Compile(ObjBlock, '-c'); 
       Compile(Block, '');
     end
     ModifyHelp(Block);
     ModifyFlouLibPath;
     ModifyFlouLibRef(Block);
     % Add Floulib to the path
     addpath(pwd);
     ModifyPathDef;
     disp('Installation completed');
  otherwise
     disp('Unknown Option');
end     


return

% Compilation
function Compile(Block,Option)

for i = 1 : length(Block)
   [pathstr,name,ext,vers] = fileparts(Block(i).FileName);
       
   switch ext
      case '.c'
         if isempty(Block(i).Obj)
            CompileStr = ['mex ', Option, ' ', name, '.c' ];
         else
            CompileStr = ['mex ', Option, ' ', name, '.obj ', ' ', Block(i).Obj];
         end   
         disp(['  -> Compiling block ',Block(i).Name]);
         evalin('base', CompileStr, 'disp([''    Error in : '', CompileStr])');
   end
end
return

% UpDate
function UpDate(Block,Option)
for i = 1 : length(Block)
    
   [pathstr,name,ext,vers] = fileparts(Block(i).FileName);

   if ~ isempty(Block(i).Obj)
      CompileStr = ['mex ', Option, ' ', name , '.obj ', ' ', Block(i).Obj];
      disp(['  -> UpDating block ',Block(i).Name]);
      evalin('base', CompileStr, 'disp([''    Error in : '', CompileStr])');
   end   
end
return

% Creation of the blocks
function Block = NewBlock(Name,Type,FileName,Obj)
Block.Name = Name;
Block.Type = Type;
Block.FileName = FileName;
Block.Obj  = Obj;
return

% Modify FlouLibPath
function ModifyFlouLibPath

fdw = fopen('FlouLibPath.m','w');
fprintf(fdw,'%% This function is created when installing FlouLib\r\n');
fprintf(fdw,'function Path=FlouLibPath\r\n');
fprintf(fdw,'%s\r\n',['Path = ''',pwd,''';']);
fprintf(fdw,'return\r\n');
fclose(fdw);
return

% Modification of the help
function ModifyHelp(Block)

disp(['  -> Installation of the help ']);

for i=1 : length(Block)
   copyfile(['.\Help\' , Block(i).Name, '.html'],'.\Help\Tempo.html');
   fdr = fopen('.\Help\Tempo.html');
   fdw = fopen(['.\Help\' , Block(i).Name, '.html'],'w');
   
   Ecriture = 1;
   while ~ feof(fdr)
      X = fgets(fdr);
   
      Res = findstr(X,'<!-- START HEADER -->');
      if ~ isempty(Res)

         fprintf(fdw,'%s',X);
         Ecriture = 0;
         % header simulink
         fds = fopen('.\Help\Simulink_Header.html');
         while ~ feof(fds)
            fprintf(fdw,'%s',strrep(fgets(fds),'<!-- MATLABROOT -->',matlabroot));
         end   
         fclose(fds);   
         % header floulib
         fds = fopen('.\Help\FlouLib_Header.html');
         while ~ feof(fds)
            fprintf(fdw,'%s',strrep(fgets(fds),'<!-- BLOCK NAME -->',Block(i).Name));
         end   
         fclose(fds);   
      end
        
      Res = findstr(X,'<!-- END HEADER -->');
      if ~ isempty(Res)
         Ecriture = 1;
      end
      
      if Ecriture
         fprintf(fdw,'%s',X);
      end
   end
   fclose(fdr);
   fclose(fdw);
end
delete('.\Help\Tempo.html');
return

% Modification of pathdef.m
function ModifyPathDef

disp(['  -> Modification of the path ']);

if ~exist([matlabroot,'\toolbox\local\copy_pathdef.m'])
copyfile([matlabroot,'\toolbox\local\pathdef.m'],[matlabroot,'\toolbox\local\copy_pathdef.m']);
end
copyfile([matlabroot,'\toolbox\local\pathdef.m'],'.\Tempo.m');
fdr = fopen('.\Tempo.m');
fdw = fopen([matlabroot,'\toolbox\local\pathdef.m'],'w');
 
while ~ feof(fdr)
   X = fgets(fdr);
   Res = findstr(X,'[...');
   if ~ isempty(Res)
      fprintf(fdw,'%s',X);
      fprintf(fdw,'%s\n',['   ''',lower(pwd),';'',...']);
   else
      Res = findstr(X,'floulib');
      if isempty(Res)
         fprintf(fdw,'%s',X);
      end
   end   
end
fclose(fdr);
fclose(fdw);
delete('.\Tempo.m');
return


% Modification of FlouLibRef.m
function ModifyFlouLibRef(Block)

[pathstr,name,ext,versn]=fileparts(floulibpath);

copyfile('.\Help\FlouLibRef.html','.\Help\Tempo.html');
fdr = fopen('.\Help\Tempo.html');
fdw = fopen('.\Help\FlouLibRef.html','w');

Ecriture = 1;
Type = '';
while ~ feof(fdr)
  X = fgets(fdr);
   
  Res = findstr(X,'<!-- START HEADER -->');
  if ~ isempty(Res)
     fprintf(fdw,'%s',X);
     Ecriture = 0;
     % header simulink
     fds = fopen('.\Help\Simulink_Header.html');
     while ~ feof(fds)
        fprintf(fdw,'%s',strrep(fgets(fds),'<!-- MATLABROOT -->',matlabroot));
     end   
     fclose(fds);   
     % header floulib
     fds = fopen('.\Help\FlouLib_Header.html');
     while ~ feof(fds)
        fprintf(fdw,'%s',strrep(fgets(fds),'<!-- BLOCK NAME -->',name));
     end   
     fclose(fds);   
  end
     
  Res = findstr(X,'<!-- END HEADER -->');
  if ~ isempty(Res)
     Ecriture = 1;
  end
  
  Res = findstr(X,'<!-- START -->');
  if ~ isempty(Res)
     fprintf(fdw,'%s',X);
     Ecriture = 0;
     fprintf(fdw,'<center><p><p>\r\n<table bgcolor="#f0f3ff" align="center" border width="90%%" cellpadding="10">\r\n');
     i = 1;
     j = 0;
     while i <= length(Block)
        if mod(j,3) == 0
           fprintf(fdw,'  <tr>\r\n');
        end   
        if ~ strcmp(Block(i).Type, Type)
           fprintf(fdw,'    <td valign="top">\r\n');
           fprintf(fdw,'      <center><b><font size="3">%s</font></b></center><br>\r\n',Block(i).Type);
           Type = Block(i).Type;
        end
        while i<=length(Block) & strcmp(Block(i).Type, Type)
           fprintf(fdw,'      <a href="%s.html">%s</a><br>\r\n',Block(i).Name,Block(i).Name);
           i = i + 1;
        end
        fprintf(fdw,'    </td>\r\n\r\n');
        j = j + 1;
        if mod(j,3) == 0
           fprintf(fdw,'  </tr>\r\n\r\n\r\n');
        end   

     end
     
     if mod(j,3) > 0
        while mod(j,3)>0
           fprintf(fdw,'    <td>&nbsp;</td>\r\n');
           j = j + 1;
        end  
        fprintf(fdw,'  </tr>\r\n\r\n');
     end   
     fprintf(fdw,'</table>\r\n');
     fprintf(fdw,'</center>\r\n');
   
  end   
  Res = findstr(X,'<!-- END -->');
  if ~ isempty(Res)
     Ecriture = 1;
  end
  if Ecriture
     fprintf(fdw,'%s',X);
  end
end
fclose(fdr);
fclose(fdw);
delete('.\Help\Tempo.html');
return

% Export Floulib
function ExportFlouLib

[pathstr,name,ext,versn]=fileparts(floulibpath);

ExportDir = ['Export_' name];
HelpDir = 'Help';
DemosDir = 'Demos';
% Create the directories
disp(['  -> Creating the export directory ']);

if ~exist(ExportDir)
   mkdir(ExportDir);
end

%Create the Help directory
disp(['  -> Creating the help directory ']);

NewDir = [ExportDir '\' HelpDir];
if ~exist(NewDir)
   mkdir(NewDir);
end
delete([ExportDir '\' HelpDir '\*.*']);
Files = dir(HelpDir);

for i = 1:size(Files)
   if(~strcmp(Files(i).name,'.') & ~strcmp(Files(i).name,'..'))
      copyfile([HelpDir '\' Files(i).name],[ExportDir '\' HelpDir '\' Files(i).name]);
   end
end

%Create the Demos directory
disp(['  -> Creating the demos directory ']);

NewDir = [ExportDir '\' DemosDir];
if ~exist(NewDir)
   mkdir(NewDir);
end

%  Delete the Export\Demos directories
Files = dir([ExportDir '\' DemosDir]);
for i = 1:size(Files)
   if(~strcmp(Files(i).name,'.') & ~strcmp(Files(i).name,'..'))
      delete([ExportDir '\' DemosDir '\' Files(i).name '\*.*']);
      eval(['!rmdir ' ExportDir '\' DemosDir '\' Files(i).name]);
   end
end

%  Create and copy the Demos directories
Files = dir([DemosDir]);
for i = 1:size(Files)
   if(~strcmp(Files(i).name,'.') & ~strcmp(Files(i).name,'..'))
      delete([ExportDir '\' DemosDir '\' Files(i).name '\*.*']);
      mkdir([ExportDir '\' DemosDir '\' Files(i).name]);
      Files_1 = dir([DemosDir '\' Files(i).name]);
      for j = 1:size(Files_1)
         if(~strcmp(Files_1(j).name,'.') & ~strcmp(Files_1(j).name,'..'))
           copyfile([DemosDir '\' Files(i).name '\' Files_1(j).name],[ExportDir '\' DemosDir '\' Files(i).name '\' Files_1(j).name]);
         end
      end  
   end
end

% Copy the dll files
Files = dir(['*.dll']);
for i = 1:size(Files)
    copyfile([Files(i).name],[ExportDir '\' Files(i).name]);
end

% Copy the obj files
Files = dir(['*.obj']);
for i = 1:size(Files)
    copyfile([Files(i).name],[ExportDir '\' Files(i).name]);
end

% Copy the h files
Files = dir(['*.h']);
for i = 1:size(Files)
    copyfile([Files(i).name],[ExportDir '\' Files(i).name]);
end

% Copy the m files
Files = dir(['*.m']);
for i = 1:size(Files)
    copyfile([Files(i).name],[ExportDir '\' Files(i).name]);
end
 
% Copy the user files
Files = dir(['User*.c']);
for i = 1:size(Files)
    copyfile([Files(i).name],[ExportDir '\' Files(i).name]);
end
 
% Copy the FlouLib.mdl file
copyfile(['FlouLib.mdl'],[ExportDir '\' 'FlouLib.mdl']);


% Create the version.txt file

[pathstr,name,ext,versn]=fileparts(floulibpath);

disp(['  -> Creating the version file : Version = ',name]);

fdw = fopen([ExportDir '\' 'Version.txt'],'w');
fprintf(fdw,'Version = %s\r\n',name);
fclose(fdw);

% Copy the README.txt file
copyfile(['README.txt'],[ExportDir '\' 'README.txt']);

return