%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Parse a FIS file
%
% Author: L. Foulloy
%
% Revision: V3.0 - L. Foulloy - 09/06/2003
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function ParseFIS(File)

fdr = fopen(File);
fdw = 0;

CptPartition = 0;

State = 0;

while ~ feof(fdr)
      
   X = fgets(fdr);
   
   if X == char(10) | X == char(13)
      State = 0;
   end
   
   if findstr(X,'[System]')
      State = 1;
      X = fgets(fdr);
      eval([X(1:length(X)-1),';']);
      ProjectName = Name;
      X = fgets(fdr);
   end
   
   if findstr(X,'[Input')
      State = 2;
      CptPartition = CptPartition + 1;
      CptMF = 0;
      X = fgets(fdr);
      eval([X(1:length(X)-1),';']);
      X = fgets(fdr);
      eval([X(1:length(X)-1),';']);
      X = fgets(fdr);
      eval([X(1:length(X)-1),';']);
      X = fgets(fdr);
      fdw = fopen([ProjectName,'_',Name,'.txt'],'w');
      fprintf(fdw,'%d\r\n',NumMFs);
      Prop(CptPartition).NbMF = NumMFs;
   end
   
   if findstr(X,'[Output')
      State = 3;
      CptPartition = CptPartition + 1;
      CptMF = 0;
      X = fgets(fdr);
      eval([X(1:length(X)-1),';']);
      X = fgets(fdr);
      eval([X(1:length(X)-1),';']);
      X = fgets(fdr);
      eval([X(1:length(X)-1),';']);
      X = fgets(fdr);
      if strcmp(Type,'mamdani')
         fdw = fopen([ProjectName,'_',Name,'.txt'],'w');
         fprintf(fdw,'%d\r\n',NumMFs);
         Prop(CptPartition).NbMF = NumMFs;
      end   
   end;
   if findstr(X,'MF')
      State = 4;
   end;
   
   if findstr(X,'[Rules]')
      if NumOutputs == 1
         fd(1).fdw = fopen([ProjectName,'_Rules.txt'],'w');
         fprintf(fd(1).fdw,'%d ',NumInputs);
         if(strcmp(Type,'mamdani'))
            fprintf(fd(1).fdw,'%d',Prop(NumInputs+1).NbMF);
         end
         fprintf(fd(1).fdw,'\r\n');
      else
         for i=1:NumOutputs
            fd(i).fdw =fopen([ProjectName,'_Rules_',num2str(i),'.txt'],'w');
            fprintf(fd(i).fdw,'%d ',NumInputs);
            if(strcmp(Type,'mamdani'))
              fprintf(fd(i).fdw,'%d',Prop(NumInputs+i).NbMF);
            end
            fprintf(fd(i).fdw,'\r\n');
         end
      end            
      State = 5;
      X = fgets(fdr);
   end;
      
   switch State
      case 0
         if fdw ~= 0
            fclose(fdw);
            fdw = 0;
         end   
      case 1
         eval([X(1:length(X)-1),';']);

      case 4
         CptMF = CptMF + 1;
         P1 = findstr(X,'=');
         P2 = findstr(X,':');
         P3 = findstr(X,',');
         NameMF = X(P1+2:P2-2);
         TypeMF = X(P2+2:P3-2);
         DefMF = eval(X(P3+1:length(X)));
         MF(CptPartition,CptMF).Name = NameMF;
         MF(CptPartition,CptMF).Def = DefMF;
         
         switch TypeMF
            case 'trimf'
               fprintf(fdw,'%s %f %f %f %f\r\n',NameMF,DefMF(1),DefMF(2),DefMF(2),DefMF(3));
            case 'trapmf'
               fprintf(fdw,'%s %f %f %f %f\r\n',NameMF,DefMF(1),DefMF(2),DefMF(3),DefMF(4));
            case 'constant'
               if(strcmp(Type,'mamdani'))
                 fprintf(fdw,'%s %f %f %f %f\r\n',NameMF,DefMF(1),DefMF(1),DefMF(1),DefMF(1));
               end;
            otherwise
               disp('This type of membership function cannot be parsed.');
               return;
         end
      case 5
         P1 = findstr(X,',');
         P2 = findstr(X,'(');
         P3 = findstr(X,')');
         P4 = findstr(X,':');

         Antecedent = eval(['[',X(1:P1-1),']']);
         Conclusion = eval(['[',X(P1+1:P2-1),']']);
         Weight = eval(X(P2+1:P3-1));
         Ope = eval(X(P4+1:length(X)));
         if Ope ~= 1
            disp('ParseFIS can only parse rules connected by "and" operator');
            return;
         end
         for i = 1:NumOutputs
            if Conclusion(i) < 0
               disp('ParseFIS cannot parse rule containing "not" operator');
               return;
            end  
            if Conclusion(i) > 0
        	      for j = 1:NumInputs
                   if Antecedent(j) < 0
                      disp('ParseFIS cannot parse rule containing "not" operator');
                      return;
                   end  
                   if Antecedent(j) > 0
                      fprintf(fd(i).fdw,'%s ',MF(j,Antecedent(j)).Name);
                   end
                   if Antecedent(j) == 0
                      fprintf(fd(i).fdw,'* ');
                   end  
               end
                     
               if strcmp(Type,'mamdani')
                  fprintf(fd(i).fdw,'%s',MF(NumInputs+i,Conclusion(i)).Name);
                  if Weight == 1
                     fprintf(fd(i).fdw,'\r\n');
                  else   
                     fprintf(fd(i).fdw,' %f\r\n',Weight);
                  end
               end
               if strcmp(Type,'sugeno')
                  fprintf(fd(i).fdw,'%f',MF(NumInputs+i,Conclusion(i)).Def);
                  if Weight == 1
                     fprintf(fd(i).fdw,'\r\n');
                  else   
                     disp('ParseFIS cannot parse Sugeno FIS containing weights less than 1');
                     return;
                  end
               end   
            end %if Conclusion(i) > 0
          end %for
      end %switch   
end %while
   
% Close files
fclose(fdr);

if fdw ~= 0
   fclose(fdw);
end
for i = 1:NumOutputs
   fclose(fd(i).fdw);
end

% Modification of the operator

% And Method
switch AndMethod
  case 'min'
     OpeTAntecedent= '1';
  case 'prod'
     OpeTAntecedent= '2';
  otherwise
     disp('ParseFIS cannot parse the "AndMethod" operator');
end

% Imp Method
switch ImpMethod
  case 'min'
     OpeT = '1';
  case 'prod'
     OpeT = '2';
  otherwise
     disp('ParseFIS cannot parse the "ImpMethod" operator');
end

% Agg Method
switch AggMethod
  case 'max'
     OpeSup = '-1';
  case 'probor'
     OpeSup = '-2';
  otherwise
     disp('ParseFIS cannot parse the "AggMethod" operator');
end

% Defuz Method
switch DefuzzMethod
  case 'som'
     DefuzMethod = '2';
     disp('Warning. It is an approximate conversion. See help');
  case 'lom'
     DefuzMethod = '3';
     disp('Warning. It is an approximate conversion. See help');
  case 'mom'
     DefuzMethod = '4';
  case 'centroid'
     DefuzMethod = '5';
otherwise
     DefuzMethod = '0';
     disp('ParseFIS cannot parse the DefuzMethod. It is replaced by the Height Method');
end

% Change Methods
if strcmp(Type, 'mamdani')
   Res = find_system(gcs,'FindAll','off');
   Size= size(Res);
   for i=1:Size(1)
      % Change Inference Operators
      if findstr(char(Res(i)),'SymbInfMam') & findstr(get_param(char(Res(i)), 'FileInf'), ProjectName)
         if get_param(char(Res(i)), 'OpeSup') ~= OpeSup
            set_param(char(Res(i)), 'OpeSup',OpeSup);
         end   
         if get_param(char(Res(i)), 'OpeT') ~= OpeT
            set_param(char(Res(i)), 'OpeT',OpeT);
         end   
         if get_param(char(Res(i)), 'OpeTAntecedent') ~= OpeTAntecedent
            set_param(char(Res(i)), 'OpeTAntecedent',OpeTAntecedent);
         end   
      end
      
      % Change Defuzzification Method
      if findstr(char(Res(i)),'SymbDefuz') & findstr(get_param(char(Res(i)), 'FileDefuz'), ProjectName)
         if get_param(char(Res(i)), 'DefuzMethod') ~= DefuzMethod
            set_param(char(Res(i)), 'DefuzMethod',DefuzMethod);
         end   
      end
      
      % Change Operators for MamdaniFLC
      if findstr(char(Res(i)),'MamdaniFLC') & findstr(get_param(char(Res(i)), 'FileInf'), ProjectName)
         if get_param(char(Res(i)), 'OpeSup') ~= OpeSup
            set_param(char(Res(i)), 'OpeSup',OpeSup);
         end   
         if get_param(char(Res(i)), 'OpeT') ~= OpeT
            set_param(char(Res(i)), 'OpeT',OpeT);
         end   
         if get_param(char(Res(i)), 'OpeTAntecedent') ~= OpeTAntecedent
            set_param(char(Res(i)), 'OpeTAntecedent',OpeTAntecedent);
         end   
         if get_param(char(Res(i)), 'DefuzMethod') ~= DefuzMethod
            set_param(char(Res(i)), 'DefuzMethod',DefuzMethod);
         end   
      end
	end
end

return

