#include <stdio.h>

#include "message.h"

#define MAX_REFERENCE 		200
#define _X              	32
#define MAX_PT_2D       	500
#define MAX_TRIANGLE_2D 	MAX_PT_2D*3
#define STRING_SIZE			100

#define Min(x,y) ((x)<(y)?(x):(y))
#define Max(x,y) ((x)>(y)?(x):(y))

//---------------------------------------------------------------------------------------------------------
// Various Types
//
typedef char String_X[_X+1];

typedef real_T GradeOfMembership;

typedef GradeOfMembership (* Operator)(GradeOfMembership,GradeOfMembership);
typedef GradeOfMembership * LinguisticFuzzySubset;

typedef struct {
   double Left;
   double Right;
   } Interval;

typedef struct {
	double Left;
	double Mode;
	double Right;
	} Triangle;

typedef struct {
	double Left;
	double LeftMode;
	double RightMode;
	double Right;
	} Trapeze;

//---------------------------------------------------------------------------------------------------------
// Membership Function
//
typedef union {
   Triangle Triangle;
   Trapeze Trapeze;
   } MembershipFunction;

//---------------------------------------------------------------------------------------------------------
// Fuzzy Meaning
//
typedef struct {
	String_X Name;
	MembershipFunction MembershipFunction;
	} FuzzyMeaning;

//---------------------------------------------------------------------------------------------------------
// Fuzzy Partition
//
typedef struct {
	int NbSymbol;
	FuzzyMeaning * TabFuzzyMeaning;
	} FuzzyPartition;

//---------------------------------------------------------------------------------------------------------
// 2D Fuzzy Partition
//
typedef struct {
   int p[3];          // Points defining the triangle
   double c[2];       // Centre of the circle
   double r;          // Radius of the circle
   } DelaunayTriangle;

typedef struct {
   struct {
      double x;
      double y;
      int NoSymbol;
      } TabPoint[MAX_PT_2D];									// Table of the points
   DelaunayTriangle TabTriangle[MAX_TRIANGLE_2D];		// Table of the triangles
   String_X * TabSymbol;										// Table of the linguistic terms
   int NbPoint;													// Number of points
   int NbTriangle;												// Number of triangles
   int NbSymbol;													// Number of symbols
   } FuzzyPartition2D;


//---------------------------------------------------------------------------------------------------------
// Function block for the fuzzification
//
typedef struct {
	FILE * File;
	int NoReference;
	FuzzyPartition FuzzyPartition;
	LinguisticFuzzySubset Result;
	} Fuzzification;

//---------------------------------------------------------------------------------------------------------
// Functional block for a 2D fuzzification
//
typedef struct {
	FILE * File;
	int NoReference;
	FuzzyPartition2D FuzzyPartition2D;
	LinguisticFuzzySubset Result;
	} Fuzzification2D;

//---------------------------------------------------------------------------------------------------------
// Functional block for a Mamdani-type symbolic (linguistic) inference
//
typedef struct {
	FILE * File;
	int NoReference;
	int NbInput;
	int NbRule;
	int NbOutputSymbol;
	GradeOfMembership ** Result; // for MamdaniFLC
	Operator OpeSup;
	Operator OpeT;
	Operator OpeTAntecedent;
	double ** TabRule;
	} MamdaniSymb;	

//---------------------------------------------------------------------------------------------------------
// Functional block for a Sugeno-type symbolic (linguistic) inference
//
typedef struct {
	FILE * File;
   int NoReference;
   int NbInput;
	int NbRule;
	double ** TabRule;
	} SugenoSymb;	

//---------------------------------------------------------------------------------------------------------
// Functional block for the defuzzification
//
typedef struct {
	double Area;    // Area of the membership function
	double Cog;     // Centre of gravity of the membership function
	double Modal;	 // Modal value of the membership function
	} Attribute;

typedef struct {
	double Min;
	double Max;
	double NbPoints;
   double Step;
   } GeneralAttribute;

typedef struct {
	FILE * File;
	int NoReference;
	int TypeDefuz;
   GeneralAttribute GeneralAttribute;
	FuzzyPartition FuzzyPartition;
	Attribute * TabAttribute;
	} Defuzzification;

//---------------------------------------------------------------------------------------------------------
// Functional block for a Sugeno-type Fuzzy Logic Controller
//
typedef struct {
	FILE * File;
	int NoReference;
	int NbInput;
   Fuzzification * TabFuzzification;
	SugenoSymb Inference;
	} SugenoFLC;

//---------------------------------------------------------------------------------------------------------
// Functional block for a Mamdani-type Fuzzy Logic Controller
//
typedef struct {
	FILE * File;
	int NoReference;
	int NbInput;
   Fuzzification * TabFuzzification;
	MamdaniSymb Inference;
   Defuzzification Defuzzification;
	} MamdaniFLC;

//---------------------------------------------------------------------------------------------------------
// Fuzzy Project
//
typedef union {
	Fuzzification Fuzzification;
	Fuzzification2D Fuzzification2D;
	MamdaniSymb MamdaniSymb;
	SugenoSymb SugenoSymb;
	Defuzzification Defuzzification;
	SugenoFLC SugenoFLC;
	MamdaniFLC MamdaniFLC;
	} FuzzyProject;

//---------------------------------------------------------------------------------------------------------
// References
//

typedef struct {
	int NbSymbol;
	String_X * TabSymbol;
	} Reference;

typedef struct {
	int NbReference;
	Reference Reference[MAX_REFERENCE];
	} TabReference;


