/*------------------------------------------------------------------------------------
UserOperators.c

This file contains the definitions of fuzzy operators that are used in several Floulib Blocks

Copyright: Universit de Savoie, LISTIC/ESIA.

Author: L. Foulloy

Revision: V3.0 - L. Foulloy - 08/06/2003
------------------------------------------------------------------------------------ */

#include "simstruc.h"
#include "type.h"

//----------------------------------------------------------------------------------------------------------
// Prototypes
//
static GradeOfMembership mini(GradeOfMembership x, GradeOfMembership y);
static GradeOfMembership maxi(GradeOfMembership x, GradeOfMembership y);

static GradeOfMembership mini(GradeOfMembership x, GradeOfMembership y);
static GradeOfMembership prod(GradeOfMembership x, GradeOfMembership y);
static GradeOfMembership Lukasiewicz_tnorm(GradeOfMembership x, GradeOfMembership y);

static GradeOfMembership maxi(GradeOfMembership x, GradeOfMembership y);
static GradeOfMembership probsum(GradeOfMembership x, GradeOfMembership y);
static GradeOfMembership Lukasiewicz_tconorm(GradeOfMembership x, GradeOfMembership y);

Operator tconorm(int x);
Operator tconorm(int x);
Operator FuzzyOperator(int x);

//----------------------------------------------------------------------------------------------------------
// Operators
//

Operator tnorm(int x)
{
  switch(x)
    {
    case 1: return(mini);
    case 2: return(prod);
    case 3: return(Lukasiewicz_tnorm);
    default: return(NULL);
    }
}

Operator tconorm(int x)
{
  switch(x)
    {
    case -1: return(maxi);
    case -2: return(probsum);
    case -3: return(Lukasiewicz_tconorm);
    default: return(NULL);
    }
}

Operator FuzzyOperator(int x)
{
  switch(x)
    {
// t_norms
    case 1: return(mini);
    case 2: return(prod);
    case 3: return(Lukasiewicz_tnorm);

// t_conorms
    case -1: return(maxi);
    case -2: return(probsum);
    case -3: return(Lukasiewicz_tconorm);

    default: return(NULL);
    }
}

//----------------------------------------------------------------------------------------------------------
// t-norms
//

static GradeOfMembership mini(GradeOfMembership x, GradeOfMembership y)
{
  if(x<y) return(x); else return(y);
}

static GradeOfMembership prod(GradeOfMembership x, GradeOfMembership y)
{
  return(x*y);
}

static GradeOfMembership Lukasiewicz_tnorm(GradeOfMembership x, GradeOfMembership y)
{
  return(maxi(x+y-1,0));
}

//----------------------------------------------------------------------------------------------------------
// t-conorms
//

static GradeOfMembership maxi(GradeOfMembership x, GradeOfMembership y)
{
  if(x>y) return(x); else return(y);
}

static GradeOfMembership probsum(GradeOfMembership x, GradeOfMembership y)
{
  return(x+y-x*y);
}

static GradeOfMembership Lukasiewicz_tconorm(GradeOfMembership x, GradeOfMembership y)
{
  return(mini(x+y,1));
}


